#include <assert.h>
#include <errno.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

#include "comb.h"

void usage(const char *argv0)
{
	fprintf(stderr, "usage: %s N K\n", argv0);
}

bool parse_number(const char *str, long long *number)
{
	assert(str != NULL);
	assert(number != NULL);
	errno = 0;

	char *endp;
	*number = strtoll(str, &endp, 10);

	return endp != str && *endp == '\0' && errno != ERANGE;
}

int main(int argc, char *argv[])
{
	if (argc != 3) {
		usage(argv[0]);
		return EXIT_FAILURE;
	}

	long long args[2];
	for (int i = 0; i < 2; ++i) {
		if (!parse_number(argv[i + 1], &args[i])) {
			fprintf(stderr, "%s: Invalid input\n", argv[i + 1]);
			return EXIT_FAILURE;
		}
	}

	long long r1 = fact(args[0]);
	long long r2 = comb(args[0], args[1]);

	if (r1 < 0 || r2 < 0) {
		fprintf(stderr, "Invalid input or result out of range\n");
		return EXIT_FAILURE;
	}

	printf("%lld\n", r1);
	printf("%lld\n", r2);
	return EXIT_SUCCESS;
}
