#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <dlfcn.h>

typedef void *(malloc_f)(size_t);
static malloc_f *libc_malloc;

typedef void *(realloc_f)(void *, size_t);
static realloc_f *libc_realloc;

typedef void (free_f)(void *);
static free_f *libc_free;

__attribute__((constructor)) static
void _init(void)
{
	fprintf(stderr, "[mem] _init %ld: ", time(0));

	if ((libc_malloc = dlsym(RTLD_NEXT, "malloc")) == NULL) {
		fprintf(stderr, "[mem] dlsym malloc: %s\n", dlerror());
		abort();
	}

	if ((libc_realloc = dlsym(RTLD_NEXT, "realloc")) == NULL) {
		fprintf(stderr, "[mem] dlsym realloc: %s\n", dlerror());
		abort();
	}

	if ((libc_free = dlsym(RTLD_NEXT, "free")) == NULL) {
		fprintf(stderr, "[mem] dlsym free: %s\n", dlerror());
		abort();
	}

	fprintf(stderr, "done\n");
}

__attribute__((destructor)) static
void _fini(void)
{
	fprintf(stderr, "[mem] _fini\n");
}

void *malloc(size_t size)
{
	fprintf(stderr, "[mem] malloc(%zu) = ", size);
	void *ret = libc_malloc(size);
	fprintf(stderr, "%p\n", ret);
	return ret;
}

void *realloc(void *ptr, size_t new_size)
{
	fprintf(stderr, "[mem] realloc(%p, %zu)\n", ptr, new_size);
	void *ret = libc_realloc(ptr, new_size);
	fprintf(stderr, "%p\n", ret);
	return ret;
}

void free(void *ptr)
{
	fprintf(stderr, "[mem] free(%p)\n", ptr);
	libc_free(ptr);
}
