#ifndef STRATEGY_H
#define STRATEGY_H

/**
 * Guess Strategy API
 *
 * \file strategy.h
 * \author Roman Lacko <xlacko1@fi.muni.cz>
 */

enum guess_result {
	GR_LESS = -1,
	GR_WIN  =  0, /* For completeness, not actually used. */
	GR_MORE =  1,
};

/**
 * Initialize the strategy with range from which the secret number is chosen.
 * Returns a pointer to internal data or NULL on failure.
 *
 * The contents of the returned data is unspecified. The caller can only pass
 * the value to calls below for the same strategy.
 *
 * \param  min      lower bound (inclusive)
 * \param  max      upper bound (inclusive)
 */
void *create(long min, long max);

/**
 * Deallocates internal state of the strategy created by \code create.
 */
void destroy(void *self);

/**
 * Asks the library for next guess.
 *
 * \return self     strategy instance
 */
long guess(void *self);

/**
 * Informs the library about the result of a guess.
 *
 * It is unspecified whether only the current strategy is notified, or all
 * strategies get this information.
 *
 * \param  self     strategy instance
 * \param  number 	the guessed number
 * \param  result   GR_LESS if the guess is too high, GR_MORE if it's too low.
 */
void wrong(void *self, long number, enum guess_result result);

#endif // STRATEGY_H
