#define _GNU_SOURCE
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

#include <getopt.h>
#include <unistd.h>

struct options {
	bool a;
	bool b;
	char *c;
	char *d;

	int argc;
	char **argv;
};

static
void usage(FILE *target, const char *pname)
{
	fprintf(target, "usage: %s [--help] [OPTIONS] [PARAMETER]...\n", pname);
}

static
void help(const char *pname)
{
	usage(stdout, pname);

	printf(
		"Example of getopt(3p) usage.\n"
		"\n"
		"Brief description of options and arguments:\n"
		"  -h, --help           Show this text and exit.\n"
		"  -a, --alpha          Set option a.\n"
		"  -b, --beta           Set option b.\n"
		"  -c, --gamma=ARG      Set option c to value ARG.\n"
		"  -d, --delta[=ARG]    Set option d to value ARG.\n"
		"\n"
		"  PARAMETER    A positional string\n"
	);
}

static
bool options_parse(int argc, char *argv[], struct options *options)
{
	static const struct option longopts[] = {
		{ .name = "help", .val = 'h' },
		{ .name = "alpha", .val = 'a' },
		{ .name = "beta", .val = 'b' },
		{ .name = "gamma", .val = 'c', .has_arg = required_argument },
		{ .name = "delta", .val = 'd', .has_arg = optional_argument },
	};

	static const char optstring[] = "habc:d::";

	int option;
	while ((option = getopt_long(argc, argv, optstring, longopts, NULL)) != -1) {
		switch (option) {
		case 'h':
			help(argv[0]);
			exit(EXIT_SUCCESS);

		case 'a':
			options->a = true;
			break;

		case 'b':
			options->b = true;
			break;

		case 'c':
			options->c = argv[optind];
			break;

		case 'd':
			options->d = argv[optind] ?: "(default)";
			break;

		default:
			usage(stderr, argv[0]);
			return false;
		}
	}

	options->argc = argc - optind + 1;
	argv[optind - 1] = argv[0];
	options->argv = &argv[optind - 1];

	return true;
}

static
void options_dump(const struct options *options)
{
	printf("Program options:\n");
	printf("  [%c] a\n", options->a ? '*' : ' ');
	printf("  [%c] b\n", options->b ? '*' : ' ');
	printf("  [%c] c %s\n", options->c ? '*' : ' ', options->c);
	printf("  [%c] d %s\n", options->c ? '*' : ' ', options->d);

	putchar('\n');

	printf("Program arguments:\n");
	for (int i = 0; i < options->argc; i++)
		printf("  [%02d] \"%s\"\n", i, options->argv[i]);
}

int main(int argc, char *argv[])
{
	struct options options = { 0 };

	if (!options_parse(argc, argv, &options))
		return EXIT_FAILURE;

	options_dump(&options);
	return EXIT_SUCCESS;
}
