#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include <err.h>

#include <readline/readline.h>
#include <readline/history.h>
/* NOTE: Link with ‹-lreadline›. */

/* This source demonstrates use of GNU ‹readline()›. Again, try to use
 * some keyboard shortcuts like ‹Up›, ‹Ctrl-W›, ‹Ctrl-R›, … */

/* This file will be used to store history. */
#define READLINE_HISTORY \
	".history"

int main(void)
{
	/* Tell the library that we will want to use history-related functions.
	 * This will initialize some state for the history. */
	using_history();

	/* Reading history file for the first time may fail if the file does not
	 * exist yet. Thus we fail only if ‹errno› is something else than ‹ENOENT›. */
	if (read_history(READLINE_HISTORY) != 0 && errno != ENOENT)
		err(EXIT_FAILURE, "Failed to read history file");

	char *line;
	while ((line = readline("> ")) != NULL) {
		printf("  [%s]\n", line);

		/* Provide a way to exit the loop cleanly. */
		if (strcmp(line, "exit") == 0 || strcmp(line, "quit") == 0) {
			free(line);
			break;
		}

		/* We may add the line to history, but we only care about meaningful
		 * lines that are not empty. */
		if (line[0] != '\0')
			add_history(line);

		free(line);
	}

	/* Before we leave, save the history to a file. */
	if (write_history(READLINE_HISTORY) != 0)
		warn("Failed to write history file");

	return EXIT_SUCCESS;
}
