#include <errno.h>
#include <locale.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <err.h>

/* This demo show how to use environment variables to let The Standard Library
 * know about our locale settings. */

/* We will borrow simplified ‹DEMO()› macro from patterns. */
#define DEMO(STATEMENT) \
	do { printf("%s\n →  ", #STATEMENT); (STATEMENT); } while (0)
#define DEMOL(STATEMENT) \
	do { printf("%s\n →  %d\n", #STATEMENT, (STATEMENT)); } while (0)

int main(int argc, char *argv[])
{
	/* In the default state, functions in the standard library ignore locales
	 * completely and behave according to pristine C standard. This state
	 * is described by a special portable locale called “C”. On POSIX systems,
	 * this locale should be identical to “POSIX” locale. */

	/* Let's test this, we are going to display some values that should depend
	 * on the locale. */
	void test(void)
	{
		printf("** Current locale: %s **\n", setlocale(LC_ALL, NULL));

		/* POSIX has a special format flag, ‹'›, that uses digits separator
		 * for a given locale. */
		DEMO(printf("%'d\n", 123456789));

		/* Sorting of strings by ‹strcoll()› is also affected by locale. */
		DEMOL(strcoll("cizrna", "chlieb"));
		DEMOL(strcoll("cizrna", "Dano Drevo"));

		/* The final example shall be localization of error messages. */
		DEMO(printf("%s\n", strerror(EAGAIN)));
	}

	/* Now, when you run the program for the first time (without arguments),
	 * the default “C” locale will be in effect, thus the functions will behave
	 * according to C without any localizations taken into account.*/

	/* Run the program now and observe its output. */

	/* Before the second run, we will continue with setting up the locale.
	 *
	 * NOTE: This is guarded by a condition that the program has an argument,
	 *       but obviously real applications set this unconditionally.
	 *       This condition is here only to make the demo more explanatory. */

	if (argc != 1) {
		/* The ‹setlocale()› standard function is able to change locale for
		 * the rest of the program. The first argument can specify various
		 * aspects of the locale (e.g. ‹LC_CTYPE›, ‹LC_MESSAGE›…) and the second
		 * argument sets the exact value for the locale (e.g. ‹cs_CZ.utf-8›).
		 *
		 * However, setting up individual locales is rarely needed. Most often
		 * you will simply need to say “copy locale settings from the program's
		 * environment”, which can be done by supplying arguments ‹LC_ALL›
		 * and ‹""›. Note that the call may fail and we should warn the user
		 * in that case, as it's possible that their locale is not set up
		 * correctly. */
		if (setlocale(LC_ALL, "") == NULL)
			warnx("Failed to set up locale");
	}

	/* Run the program again, but this time add an extra argument.
	 * Check the “Current locale” string in the output, and how the output
	 * changed.
	 *
	 * Now try some different other locales. See the list of supported
	 * locales on your platform by command
	 *
	 * 		$ locale -a
	 *
	 * In case you don't have enough locales, try this program on Aisa.
	 *
	 * Try running the program with ‹cs_CZ.utf-8› or ‹sk_SK.utf-8› locale,
	 * and some exotic locales like ‹ja_JP.utf-8›:
	 *
	 * 		$ LC_ALL=cs_CZ.utf-8 ./locale x
	 * */

	test();
	return EXIT_SUCCESS;
}
