#ifndef STRVEC_H
#define STRVEC_H

#include <stdbool.h>
#include <stddef.h>

/** String vector. */
struct strvec {
	/* In this declaration, we disallow the user to change these attributes… */
	const size_t size; /**< Number of elements stored. */
	const size_t capacity; /**< Maximum number of elements. */
	/* … but we don't care whether they modify these strings. */
	char *data[]; /**< (FAM) Pointers to strings stored in the vector. */
};

/** Creates an instance of the vector.
 *  \returns \c true on success, \c false on failure. */
bool strvec_create(struct strvec **vecptr);

/** Deallocates resources associated with the vector.
 *
 *  \note Does not deallocate strings in the vector. Use \c strvec_clean() for
 *        this purpose. */
void strvec_destroy(struct strvec **vecptr);

/** Removes and deallocates all strings in the vector. */
void strvec_clean(struct strvec **vecptr);

/** Inserts a new element to the end of the vector.
 *
 *  \param str String to store.
 *  \returns \c true on success, \c false on failure. */
bool strvec_push(struct strvec **vec, char *str);

/** Removes and returns the last element of the vector.
 *
 *  \returns \c NULL if the vector is empty, a pointer to string otherwise. */
char *strvec_pop(struct strvec **vec);

typedef void (strvec_cb)(char *str);

/** Calls the callback for every element in the vector. */
void strvec_for_each(const struct strvec *vec, strvec_cb *cb);

#endif // STRVEC_H
