#include <errno.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>

/* NOTE: The workings of this program are explained in the ‹translations.adoc›
 *       document. Here only the basics will be shortly described. */

#include <err.h>
#include <libintl.h>

/* The ‹_› macro is used instead of ‹gettext()›, as that makes the tagging
 * of translatable strings less intrusive:
 *
 * 		puts(gettext("Hello!"));
 * 	→	puts(_("Hello!"));
 * */
#define _ gettext

/* Similarly for ‹ngettext()›, which is, for some reason, not used very much. */
#define C_ ngettext

/* It is often useful to specify the domain as a macro to avoid repetition. */
#define L10N_DOMAIN "greeting"

void usage(const char *argv0)
{
	/* Here is an example of tagged message for ‹gettext()›. */
	printf(_("usage: %s [N]\n"), argv0);
}

int main(int argc, char *argv[])
{
	/* The usual incantation to make standard functions aware of localizations. */
	if (setlocale(LC_ALL, "") == NULL)
		warnx("Cannot setup locale");
	/* The following call creates a domain ‹"greeting"› and tells GNU GetText
	 * that ‹./locale› provides translations for this domain. */
	if (bindtextdomain(L10N_DOMAIN, "./locale") == NULL)
		warn("Cannot bind message domain");
	/* We set ‹"greeting"› as the default domain for messages in this program. */
	if (textdomain(L10N_DOMAIN) == NULL)
		warn("Cannot set default message domain");

	if (argc > 2) {
		usage(argv[0]);
		return EXIT_FAILURE;
	}

	printf(_("Hello!\n"));

	if (argc == 2) {
		char *endp;
		errno = 0;
		long n = strtol(argv[1], &endp, 10);

		if (endp == argv[1] || *endp != '\0' || errno != 0 || n < 0) {
			if (errno == 0)
				errno = EINVAL;
			err(EXIT_FAILURE, _("Cannot convert argument %s"), argv[1]);
		}

		/* Here is an example of ‹ngettext()› message.
		 * This function takes three arguments:
		 *
		 * 		ngettext(SINGULAR, PLURAL, N);
		 *
		 * which tells how a singular and a plural form look like, and chooses
		 * one of them depending on N.
		 *
		 * When translated, ‹ngettext()› may actually choose from more options
		 * in languages that have more plural forms (e.g. Czech or Slovak). */
		printf(C_("You have %ld notification\n", "You have %ld notifications\n", n), n);
	}

	return EXIT_SUCCESS;
}
