#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include <err.h>
#include <pthread.h>

/* This is not a programming example, but an investigative one.
 *
 * It is a slightly simplified and modified version of ‹ex01t›, this time
 * with four threads. Threads with even numbers add 1 to a shared variable,
 * other threads remove 1. All threads use locks, and yet, the result
 * is not zero.  */

/* TODO:
 * Read the code below and find out why it doesn't work, even though
 * locks are being held when modifying the value. */

static
void *thread_run(void *data);

#ifndef THREADS_COUNT
#	define THREADS_COUNT 6
#endif

/* This is a C11 feature. Do not worry about this, it only checks in
 * compilation time that a condition holds, and aborts the compilation if not.
 * It is also available as ‹static_assert› from assert.h, which will become
 * a keyword in C23. */
_Static_assert(THREADS_COUNT % 2 == 0, "THREADS_COUNT must be even");
_Static_assert(THREADS_COUNT < 100, "THREADS_COUNT is too large");


/* Main
 * ---- */

/* The structure to pass to threads. */
struct thread_data {
	pthread_t thread;
	pthread_mutex_t mutex;
	int value; /* Either +1 or -1. */
	volatile long *balance; /* Total balance. */
};

int main(int argc, char *argv[])
{
	/* For simplicity we will not use arguments here. */
	if (argc != 1)
		errx(EXIT_FAILURE, "This program takes no arguments");

	/* Remember: VLAs should NOT be used like this. This is just a demo
	 * and we have used ‹_Static_assert› to put an boundary on its size. */
	struct thread_data thread_data[THREADS_COUNT];

	/* Initialize the mutex. */
	pthread_mutex_t mutex;
	if ((errno = pthread_mutex_init(&mutex, NULL)) != 0)
		err(EXIT_FAILURE, "Cannot create mutex");

	volatile long balance = 0;

	/* Fill in structures for workers and start threads. */
	for (int i = 0; i < THREADS_COUNT; ++i) {
		thread_data[i] = (struct thread_data) {
			.mutex = mutex,
			/* We have even number of threads, so we create balanced number of
			 * increment and decrement workers. We should get 0 in the end. */
			.value = (i % 2 == 0) ? +1 : -1,
			.balance = &balance,
		};

		if ((errno = pthread_create(&thread_data[i].thread, NULL, thread_run,
						&thread_data[i])) != 0)
			err(EXIT_FAILURE, "Cannot start thread %d", i);
	}

	/* Join threads and print out the result. */
	for (int i = 0; i < THREADS_COUNT; ++i) {
		int pterrno = pthread_join(thread_data[i].thread, NULL);
		assert(("pthread_join failed", pterrno == 0));
	}

	printf("Balance:\t%ld\n", balance);

	pthread_mutex_destroy(&mutex);
	return 0;
}


/* Worker
 * ------ */

static
void *thread_run(void *data_raw)
{
	struct thread_data *data = data_raw;

	for (int i = 0; i < 1000; ++i) {
		if ((errno = pthread_mutex_lock(&data->mutex)) != 0)
			err(EXIT_FAILURE, "Cannot lock mutex");
		/* Critical section begins here. */

		*(data->balance) += data->value;

		/* Critical section ends here. */
		int pterrno = pthread_mutex_unlock(&data->mutex);
		assert(("pthread_mutex_unlock failed", pterrno == 0));
	}

	return NULL;
}
