#include <assert.h>
#include <errno.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <err.h>
#include <pthread.h>
#include <unistd.h>

/* TODO:
 *  1. Inspect the code below.
 *  2. Insert read-write locking in proper places to remove race conditions.
 *
 *  Do «not» change other functionality, especially do not remove sleeps. */

/* High-level overview: We have two kinds of threads here:
 * • The “filler” thread chooses a random character and fills the shared
 *   memory with it.
 * • The “announcer” thread checks that the memory contains the same byte,
 *   and if it does, prints it out.
 *
 * There are various ‹usleep()› calls in this code to make synchronization
 * errors more probable. Do not remove them. */

#define FILLERS 2
#define ANNOUNCERS 4
#define THREADS_COUNT (FILLERS + ANNOUNCERS)

struct thread_data {
	pthread_t thread;
	size_t memory_size;
	char *memory;
};

static
void *_filler(void *data_raw)
{
	struct thread_data *data = data_raw;

	while (true) {
		usleep(5000);

		char c = 'a' + rand() % ('z' - 'a' + 1);
		for (int i = 0; i < data->memory_size; ++i) {
			usleep(10);
			data->memory[i] = c;
		}
	}
}

static
void *_announcer(void *data_raw)
{
	struct thread_data *data = data_raw;

	char saved = 0;
	while (true) {
		usleep(100);

		if (saved == data->memory[0])
			continue;

		saved = data->memory[0];
		for (int i = 0; i < data->memory_size - 1; ++i) {
			usleep(10);

			if (saved != data->memory[i]) {
				warnx("%d: Corrupted data at %d (%c vs %c)", gettid(), i, saved, data->memory[i]);
				continue;
			}
		}

		printf("ok %s\n", data->memory);
		usleep(1000000);
	}
}

int main(void)
{
	char string[80] = { 0 };

	struct thread_data thread_data[THREADS_COUNT];
	for (int i = 0; i < THREADS_COUNT; ++i) {
		thread_data[i] = (struct thread_data) {
			.memory = string,
			.memory_size = sizeof(string),
		};

		if ((errno = pthread_create(&thread_data[i].thread, NULL,
						i < FILLERS ? _filler : _announcer, &thread_data[i])) != 0)
			err(EXIT_FAILURE, "Cannot create thread %d", i);
	}

	/* We will run the program for 10 seconds and then tear it down.
	 * Just to keep it as short as possible. */
	alarm(10);
	pthread_exit(NULL);
}
