#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <err.h>

#include "queue.h"

int main(int argc, char *argv[])
{
	struct queue *queue;

	int qerrno;
	if ((qerrno = queue_create(&queue, sizeof(int), 16)) != QUEUE_SUCCESS)
		errx(EXIT_FAILURE, "Cannot create queue: %s", queue_strerror(qerrno));

	/* TODO: Write more thorough tests.
 	*
 	* Tip: Make sure your queue can work with different threads.
 	*      Use ‹helgrind› to pinpoint potential problems. */

	for (int i = 0; i < 16; ++i) {
		int job_id = getpid() + 2 * i;
		if ((qerrno = queue_push(queue, &job_id)) != QUEUE_SUCCESS)
			warnx("Cannot push an element: %s", queue_strerror(qerrno));

		printf("Push %d\n", job_id);
	}

	printf("Queue size: %zu\n", queue_size(queue));

	for (int i = 0; i < 16; ++i) {
		int expected = getpid() + 2 * i;
		int fetched;

		if ((qerrno = queue_pop(queue, &fetched)) != QUEUE_SUCCESS)
			warnx("Cannot fetch an element: %s", queue_strerror(qerrno));

		printf("Fetch %d\n", fetched);
		assert(expected == fetched);
	}

	if ((qerrno = queue_destroy(queue)) != QUEUE_SUCCESS)
		warnx("Cannot destroy queue: %s", queue_strerror(qerrno));

	return EXIT_SUCCESS;
}
