#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include <err.h>
#include <unistd.h>

#ifndef BUFFER_SIZE
#	define BUFFER_SIZE 64
#endif

/* This example simply copies input stream to output stream. */

int main(void)
{
    static char buffer[BUFFER_SIZE];

    ssize_t rd = 0;
	ssize_t wt = 0;

	/* To specify standard input descriptor, we can either use ‹0›
	 * or we can use ‹STDIN_FILENO› macro for clarity. There are analogous
	 * macros for ‹STDOUT_FILENO› and ‹STDERR_FILENO›. */
    while ((rd = read(STDIN_FILENO, buffer, BUFFER_SIZE)) > 0) {
        char *ptr = buffer;

		/* Remember, ‹read()› can always return less bytes than we requested,
		 * even under normal mode of operation. The same, unfortunately, goes
		 * for ‹write()›. */
        while (rd > 0 && (wt = write(STDOUT_FILENO, ptr, rd)) > 0) {
			/* We keep the number of remaining bytes in ‹rd›. */
            rd -= wt;

			/* To avoid copying the same part of the buffer, we simply
			 * increase the pointer by the number of bytes written. */
            ptr += wt;
        }

        if (wt == -1)
            err(EXIT_FAILURE, "write");
    }

    if (rd == -1)
        err(EXIT_FAILURE, "read");

	/* Standard input and output were given to us, we do not necessarily
	 * need to close them explicitly. But we could, if we want. */

    return EXIT_SUCCESS;
}
