#include <assert.h>
#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <wait.h>


/* This is essentially the same example as before, except this time
 * the child will invoke another process (‹ls›) which will attempt to write
 * to its standard output. We want to redirect this output to the pipe
 * so that the parent can read the output. */

/* Forward declarations
 * -------------------- */

void parent(int pipefds[2]);
void child(int pipefds[2]);


/* Main
 * ---- */

int main(void)
{
	int pipefds[2];

	if (pipe(pipefds) == -1)
		err(EXIT_FAILURE, "pipe");

	pid_t pid = fork();

	switch (pid) {
	case -1:
		err(EXIT_FAILURE, "fork");

	case 0:
		child(pipefds);
		exit(EXIT_SUCCESS);
	}

	parent(pipefds);
	wait(NULL);

	return EXIT_SUCCESS;
}

void parent(int pipefds[2])
{
	/* The parent process in this example is literally the same as in
	 * ‹ex05t-pipe›. */
	close(pipefds[1]);

	char buffer;

	int rd;
	while ((rd = read(pipefds[0], &buffer, 1)) > 0)
		write(STDOUT_FILENO, &buffer, 1);

	write(STDOUT_FILENO, "\n", 1);

	if (rd == -1)
		err(EXIT_FAILURE, "read");

	close(pipefds[0]);
}


void child(int pipefds[2])
{
	/* The child is now tasked with replacing its standard output with
	 * the writing end of the pipe. Afterwards it can execute a new program,
	 * which will simply write its output into the pipe. */
	close(pipefds[0]);

	/* This call will «atomically» close standard output and then it will
	 * create a new descriptor in its place, that will point to the same
	 * OFD as ‹pipefds[1]›, which is the pipe. */
	if (dup2(pipefds[1], STDOUT_FILENO) == -1)
		err(EXIT_FAILURE, "dup2");

	/* Now we can execute ‹ls›. */
	execlp("ls", "ls", "-l", "/", NULL);
	err(EXIT_FAILURE, "execlp");
}
