#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include <err.h>
#include <ftw.h>
#include <unistd.h>

#define SGR_RED 31
#define SGR_BLUE 34
#define SGR_YELLOW 33
#define SGR_CYAN 36

#if defined(__GNUC__)
#	define unused __attribute__((unused))
#else
#	define unnused
#endif

/* This example demonstrates use of File Tree Walk API.
 * The program traverses given tree and prints names of objects it sees. */


/* Forward declarations
 * -------------------- */
int callback(const char *path, const struct stat *sb, int tf, struct FTW *ftwb);


/* Main
 * ---- */

int main(int argc, char *argv[])
{
	if (argc != 2) {
		fprintf(stderr, "usage: %s DIRECTORY\n", argv[0]);
		return EXIT_FAILURE;
	}

	/* The call to ‹nftw()› is basically all we need. The flags passed mean
	 *
	 * ◦ ‹FTW_CHDIR›: Change to a directory before handling its contents.
	 * ◦ ‹FTW_MOUNT›: Do not traverse to other file systems.
	 * ◦ ‹FTW_PHYS›: Do not follow symbolic links. */
	if (nftw(argv[1], callback, 256, FTW_CHDIR | FTW_MOUNT | FTW_PHYS) != 0)
		err(EXIT_FAILURE, "ntfw");

	/* And that's it. */

	return EXIT_SUCCESS;
}

/* The callback function is called for every object ‹nftw()› finds. */
int callback(const char *path, unused const struct stat *sb, int tf,
		struct FTW *ftwb)
{
	/* The ‹FTW› structure contains information about our current position
	 * in the file system. E.g. ‹level› tells us how many directories are there
	 * between us and the top-level directory. */
	printf("%*s", ftwb->level * 4, "");

	/* We are provided with ‹stat()› result in ‹sb›, but some information
	 * (mode to be precise) is also extracted into ‹tf›. */
	char color = '0', type;
	switch (tf) {
	case FTW_F: /* Regular file */
		type  = 'f';
		break;

	case FTW_DP: /* Directory */
	case FTW_D:
		type  = 'd';
		color = SGR_BLUE;
		break;

	case FTW_DNR: /* Unreadable directory */
		type  = 'D';
		color = SGR_YELLOW;
		break;

	case FTW_SL: /* Symbolic link (FTW_PHYS) */
		type  = 'l';
		color = SGR_CYAN;
		break;

	case FTW_SLN: /* Invalid symbolic link */
		type  = 'L';
		color = SGR_RED;
		break;

	case FTW_NS: /* Files where ‹stat()› failed */
		type  = '?';
		color = SGR_YELLOW;
		break;
	}

	printf("%c \x1b[%dm%s\x1b[0m\n", type, color, path + ftwb->base);
	return 0;
}
