#include "events.h"

#include <stdbool.h>
#include <stdio.h>
#include <string.h>

#include <sys/inotify.h>

/* NOTE: This is just an auxiliary file to hide non-important details.
 *       Read ‹inotify.c› first. */


/* X-Macros will help us to avoid repetition of code that will test bits
 * and append strings to the buffer. */
#define EXPAND_EVENTS(EXPAND) \
	EXPAND(IN_ACCESS) \
	EXPAND(IN_ATTRIB) \
	EXPAND(IN_CLOSE_WRITE) \
	EXPAND(IN_CLOSE_NOWRITE) \
	EXPAND(IN_CREATE) \
	EXPAND(IN_DELETE) \
	EXPAND(IN_DELETE_SELF) \
	EXPAND(IN_MODIFY) \
	EXPAND(IN_MOVE_SELF) \
	EXPAND(IN_MOVED_FROM) \
	EXPAND(IN_MOVED_TO) \
	EXPAND(IN_OPEN) \
	EXPAND(IN_IGNORED) \
	EXPAND(IN_ISDIR) \

const char *events_to_str(uint32_t events)
{
	/* We have 16 events, the longest has 16 bytes. We should need a bit more
	 * space for separators, but since all events cannot occur in the same
	 * event, 256 B should be enough for all possible events.
	 *
	 * Of course, if someone crafts an event mask with all events, things
	 * will go awry. */
	static char _str[256];
	sprintf(_str, "(0x%08x)", events);

	/* This flag will help us to skip the very first separator. */
	bool more = false;

	for (uint32_t bit = 0; bit < 32; ++bit) {
		const char *name = NULL;

		switch (events & (1 << bit)) {
			/* Event names will expand to switch cases, where we simply
			 * set ‹name› to name of the event. */
#define EXPAND(NAME) \
		case (NAME): name = #NAME; break;
		EXPAND_EVENTS(EXPAND)
#undef EXPAND
		}

		if (name != NULL) {
			/* This condition will be false when evaluated the first time, and
			 * after that it will always be true. It's horrible, but short. */
			if (more || !(more = true))
				strcat(_str, " |");

			strcat(_str, " ");
			strcat(_str, name);
		}
	}

	return _str;
}
