#include <inttypes.h>
#include <limits.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <err.h>
#include <sys/inotify.h>
#include <unistd.h>

#include "events.h"

/* This is basically the same program as ‹ex03t-inotify›. For your convenience,
 * the event processing was moved to ‹process_event()› function just below.
 * Your task is to modify the program. */

/* TODO: Change the code below and implement two new functionalities:
 *
 *  1. Fix ‹process_events()› so that it processes «all» events returned by
 *     the kernel, not just the first one.
 *
 *  2. Fix ‹main()› so that it will stop processing events and exit the program
 *     when the user closes the standard input (e.g. by pressing ‹Ctrl-D›).
 *     Bytes read from the input shall be ignored.
 *
 * The program should not block on any ‹read()›, especially not from the
 * standard input. On the other hand, it should be efficiently waiting without
 * overloading a CPU core. (Yes, you are supposed to use ‹poll()›). */

static
bool process_events(int in_fd)
{
	static char buffer[sizeof(struct inotify_event) + NAME_MAX + 1];

	ssize_t rd;
	if ((rd = read(in_fd, buffer, sizeof(buffer))) > 0) {
		struct inotify_event *event = (struct inotify_event *) buffer;

		printf("Inotify event:\n");
		printf(" ◦ mask: %s\n", events_to_str(event->mask));

		if (event->len > 0)
			printf(" ◦ name: %s\n", event->name);

		if (event->cookie != 0)
			printf(" ◦ cookie: %04" PRIx32 "\n", event->cookie);

		if (sizeof(*event) + event->len < rd) {
			warnx("Events dropped (used %zd, read %zd)",
					sizeof(*event) + event->len, rd);
		}
	}

	return rd >= 0;
}

int main(int argc, char *argv[])
{
	if (argc > 2) {
		fprintf(stderr, "usage: %s [DIRECTORY]\n", argv[0]);
		return EXIT_FAILURE;
	}

	const char *path = argc == 1 ? "." : argv[1];

	int in_fd = inotify_init();
	if (in_fd == -1)
		err(EXIT_FAILURE, "inotify_init()");

	if (inotify_add_watch(in_fd, path, IN_ALL_EVENTS) == -1)
		err(EXIT_FAILURE, "inotify_add_watch()");

	while (true) {
		if (!process_events(in_fd))
			err(EXIT_FAILURE, "process_events()");
	}
}
