#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <err.h>
#include <sys/wait.h>
#include <unistd.h>

/* The program below ‹fork()›s and uses child process to “compute” a single
 * 64bit value. The value is then passed to the parent via pipe, who is waiting
 * for the value on ‹read()›.
 *
 * TODO: Change this program so that it uses ‹eventfd()› instead of ‹pipe()›. */

static
int child(int fd)
{
	srand(time(NULL));

	uint64_t n = 3 + rand() % 5;
	printf("child: %" PRIu64 "\n", n);

	/* Simulate long computation using ‹sleep(n)›. */
	sleep(n);

	if (write(fd, &n, sizeof(n)) == -1)
		err(EXIT_FAILURE, "Failed to write response to parent");

	return 0;
}

static
int parent(int fd)
{
	uint64_t n;
	if (read(fd, &n, sizeof(n)) == -1)
		err(EXIT_FAILURE, "Failed to read the number");

	printf("parent: %" PRIu64 "\n", n);
	return 0;
}

int main(void)
{
	int fdpipe[2];
	if (pipe(fdpipe) == -1)
		err(EXIT_FAILURE, "pipe()");

	pid_t pid = fork();
	if (pid == -1)
		err(EXIT_FAILURE, "fork()");

	if (pid == 0) {
		/* Child process. */
		close(fdpipe[0]);
		exit(child(fdpipe[1]));
	} else {
		/* Parent process. */
		close(fdpipe[1]);
		parent(fdpipe[0]);
	}

	close(fdpipe[0]);
	if (waitpid(pid, NULL, 0) != pid)
		err(EXIT_FAILURE, "Failed to wait for the child");

	return 0;
}
