#ifndef CONFIG_H
#define CONFIG_H

#include <limits.h>

/* Page size can be a constant in ‹limits.h›, but it is not required. In that
 * case we will define ‹PAGESIZE› as ‹sys/user.h›'s ‹PAGE_SIZE›. This is not
 * portable, but works. A portable way would be to call ‹sysconf(_SC_PAGESIZE›,
 * but using function calls inside macros is not very healthy. */
#ifndef PAGESIZE
#	include <sys/user.h>
#	define PAGESIZE PAGE_SIZE
#endif // PAGESIZE

/* The size of file to create, in bytes. */
#ifndef FILESIZE
#	define FILESIZE (50 * 1024 * 1024)
#endif // FILESIZE

/* How many pages at once we want to write in one asynchronous task.
 * This value is only used in ‹aio.c›. */
#ifndef PAGES_PER_TASK
#	define PAGES_PER_TASK 1
#endif // PAGES_PER_TASK

/* Sanity checks. */
_Static_assert((FILESIZE % PAGESIZE == 0),
		"FILESIZE must be a multiple of PAGESIZE");
_Static_assert((FILESIZE / PAGESIZE) % PAGES_PER_TASK == 0,
		"PAGES_PER_TASK * PAGESIZE must divide FILESIZE without remainder");

#endif // CONFIG_H
