#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <aio.h>
#include <bsd/sys/time.h>
#include <err.h>
#include <fcntl.h>
#include <unistd.h>

/* NOTE: Check out ‹aio.c› first.
 *
 * This program does the same thing as ‹aio.c›, except directly using ‹write()›. */

#ifndef PAGESIZE
#	define PAGESIZE sysconf(_SC_PAGESIZE)
#endif // PAGESIZE

#ifndef FILESIZE
#	define FILESIZE (50 * 1024 * 1024)
#endif // FILESIZE

int main(int argc, char *argv[])
{
	if (argc != 2) {
		fprintf(stderr, "usage: %s FILE\n", argv[0]);
		return EXIT_FAILURE;
	}

	int fd = open(argv[1], O_WRONLY | O_CREAT | O_EXCL, 0600);
	if (fd == -1)
		err(EXIT_FAILURE, "%s", argv[1]);

	size_t pagesize = PAGESIZE;
	char *block = malloc(pagesize);

	if (block == NULL)
		err(EXIT_FAILURE, "Cannot allocate block");

	for (size_t i = 0; i < sizeof(block); ++i)
		block[i] = i % 256;

	size_t task_count = FILESIZE / pagesize;

	struct timespec start, finish;
	clock_gettime(CLOCK_MONOTONIC, &start);

	for (unsigned task = 0; task < task_count; ++task) {
		size_t written = 0, wr;
		while (written < pagesize
				&& (wr = write(fd, block + written, pagesize - written)) > 0) {
			written += wr;
		}

		if (wr <= 0)
			err(EXIT_FAILURE, "write()");
	}

	clock_gettime(CLOCK_MONOTONIC, &finish);

	struct timeval tv_start, tv_finish, tv_diff;
	TIMESPEC_TO_TIMEVAL(&tv_start, &start);
	TIMESPEC_TO_TIMEVAL(&tv_finish, &finish);
	timersub(&tv_finish, &tv_start, &tv_diff);
	printf("Direct write finished in %ld.%06lds\n", tv_diff.tv_sec, tv_diff.tv_usec);

	return EXIT_SUCCESS;
}
