#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <err.h>
#include <sys/socket.h>
#include <sys/un.h>


/* This is a reader part of a connection-less socket example.
 * The program creates and binds a socket that can receive data. */


int main(void)
{
	/* We will use ‹SOCK_DGRAM› to preserve message boundaries. */
	int sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock == -1)
		err(EXIT_FAILURE, "socket");

	/* Before we can receive data on a socket, we must give it a name
	 * that can be then referred to by clients. This will create a named
	 * socket. Linux also supports abstract sockets, where we would have
	 * ‹\0› as the first byte of ‹sun_path›. */
	struct sockaddr_un addr = {
		.sun_family = AF_UNIX,
		.sun_path = "socket",
	};

	/* The ‹bind()› system call binds the socket to the given UNIX address.
	 * When this call succeeds, another process can send data to this socket
	 * by using the same name for ‹sendto()›. */
	if (bind(sock, (struct sockaddr *) &addr, sizeof(addr)) == -1)
		err(EXIT_FAILURE, "bind");

	char buffer[64];

	/* To read data from the socket, we use ‹recvfrom()›. In this form,
	 * it is equivalent to ‹recv()› call, because there is no way for us
	 * to receive an address of the sender. Only ‹SOCK_STREAM› and
	 * ‹SOCK_SEQPACKET› (or bound client sockets) can get us an address for
	 * reply.
	 *
	 * You may try that yourself. Change the code below to retrieve client
	 * address:
	 *
	 * 1. Declare a variable of type ‹struct sockaddr_storage› and ‹socklen_t›.
	 * 2. Pass these variables as last two arguments of ‹recvfrom()›.
	 * 3. Print the length of received socket.
	 *
	 * You should see 0, thus, no address. */
	ssize_t rd;
	while ((rd = recvfrom(sock, buffer, sizeof(buffer), 0, NULL, NULL)) >= 0)
		printf("%.*s\n", (int) rd, buffer);

	/* Continue to ‹writer.c›. */
	return EXIT_SUCCESS;
}
