#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <err.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>

/* NOTE: Read ‹reader.c› first! */


/* This program is the writer that will connect to a socket create by
 * ‹reader.c› and will then relay data from standard input into the socket. */


/* This auxiliary function removes newlines from the input. */
static
void trim(char *str)
{
	char *nl = strchr(str, '\n');
	if (nl != NULL)
		*nl = '\0';
}

int main(void)
{
	/* We create the socket and its address the same way as for ‹reader.c›. */
	int sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock == -1)
		err(EXIT_FAILURE, "socket");

	struct sockaddr_un server = {
		.sun_family = AF_UNIX,
		.sun_path = "socket",
	};

	/* This time we do not use ‹bind()›, as we are going to send data. */
	char buffer[64];
	while (fgets(buffer, sizeof(buffer), stdin) != NULL) {
		trim(buffer);

		/* Using ‹sendto()› with the address of the target socket, we pass
		 * the buffer to the other process. */
		if (sendto(sock, buffer, sizeof(buffer), 0,
					(struct sockaddr *) &server, sizeof(server)) == -1)
			err(EXIT_FAILURE, "sendto");
	}

	/* Connectionless socket does not allow reader nor writer to pass
	 * a response. The writer knows the sender's address, but since it doesn't
	 * have a bound socket, it cannot use ‹recvfrom()›. This is the reason
	 * why reader does not get an address of the writer, and thus, cannot
	 * send a reply. */

	/* To allow this kind of communication, we have to either use abstract
	 * sockets or true connection-based protocols. */

	return EXIT_SUCCESS;
}
