#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <arpa/inet.h>
#include <err.h>
#include <getopt.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>

/* When ‹get*info()› functions fail, they return their own error codes and
 * do not touch ‹errno›, except for ‹EAI_SYSTEM›, which indicates that some
 * POSIX C Library function failed. To make your code simpler, this function
 * deals with both cases. Use it like this:
 *
 * 		int neterrno;
 * 		if ((neterrno = get*info(…)) != 0)
 * 			net_error(‹0 or EXIT_FAILURE›, neterrno);
 * */
void net_error(int exit_status, int neterrno)
{
	if (neterrno == EAI_SYSTEM)
		warn("Name resolution failed");
	else
		warnx("Name resolution failed: %s", gai_strerror(neterrno));

	if (exit_status != 0)
		exit(exit_status);
}

int main(int argc, char *argv[])
{
	if (argc != 2)
		errx(EXIT_FAILURE, "usage: %s <HOST|IP>", argv[0]);

	/* TODO:
	 * 1. If HOST is given, resolve the argument into available addresses.
	 *
	 *    a. First, use ‹getaddrinfo()› to obtain addresses for the given host. */


	/*    b. For each found address, use ‹getnameinfo()› with ‹NI_NUMERICHOST›
	 *       flag to convert binary IP address in ‹sa_sockaddr*› structure to
	 *       a string. */


	/* 2. If IP address is given, use reverse resolution to get a list of
	 *    host names assigned to this address.
	 *
	 *    a. To use a reverse resolution, you need to fill an appropriate
	 *       ‹sa_sockaddr*› structure for the given address type.
	 *
	 *       Luckily, ‹getaddrinfo()› can do just that -- if you supply
	 *       ‹host› in IP dot notation (or colon notation for IPv6), no lookup
	 *       is performed, but you will get parsed addresses instead. */


	/*   b.  Once you have ‹sa_sockaddr*› structures ready, use ‹getnameinfo()›
	 *       to reverse resolve the addresses back to host names. */


	/* TIP: As you can see, both code paths above require you to call
	 *      ‹getaddrinfo()› followed by ‹getnameinfo()›. Though both paths
	 *      may require different flags, this can be used to simplify the code
	 *      a little bit. */
	return EXIT_SUCCESS;
}
