#ifndef DEMO_H
#define DEMO_H

/* Various arcane magic and sorcery hidden from demo files.
 * You are advised to close this file and never open it again. */


/* Seriously. */


/* But if you insist… I will briefly describe what's going on. */


/* First, we will most likely need the following headers in each demo.
 * Yes, hiding header files like this is generally an awful thing to do.
 * The point here, though, is to make demos shorter and plain. */
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>

#include <errno.h>
#include <err.h>

/* This magic construct tells GCC to imagine that ‹-Wno-format› option is
 * in effect from now on. «DO NOT» use this kind of pragmas in production
 * code. They actually hide potential errors! This is only here to make
 * GCC quiet about ‹DEMO()› macro below. */
#pragma GCC diagnostic ignored "-Wformat"

/* This magical macro (using various GNU extensions) is here to avoid code
 * repetition. It will print the ‹CALL› and current line with its output,
 * while still evaluating to the value of ‹CALL›.
 *
 * It relies on two GNU C Compiler extensions:
 * • Automatic types (in C++ you would use ‹auto› for this)
 * • Statement expressions (syntax extension that allows to put statements
 *   in places where expressions are expected)
 *
 * In addition, to select correct format specifier, C11's ‹_Generic()›
 * compile-time selection is employed.
 *
 * It's just so wrong on so many levels. */
#define DEMO(CALL) ({ \
		__auto_type _rv = (CALL); \
		printf(_Generic((_rv), \
				void*: "%3d  %s -> %p\n", \
				char*: "%3d  %s -> \"%s\"\n", \
				unsigned: "%3d  %s -> %u\n", \
				unsigned long: "%3d  %s -> %lu\n", \
				default: "%3d  %s -> %d\n"), \
				__LINE__, #CALL, _rv); \
		_rv; \
	})

/* The same, except without printing the return value. */
#define DEMO_NV(CALL) \
	do { \
		printf("%3d  %s\n", __LINE__, #CALL); \
		(CALL); \
	} while (0)

/* These are just some placeholder macros for demo constructs. */
#define CONDITION1 1

#endif // DEMO_H
